#
# ScriptSite.py
#

# part of ScriptingCMF (version 0.1)
# copyright: ITB, Humboldt-University Berlin, 2003
# written by Raphael Ritz, September 2003

# This module is to be placed in INSTANCE_HOME/Extensions
# where all sources of External Methods should go.

# For 'script_site' to be executable you need to have Zope
# and CMF, and optionally Plone, CMFBibliography,
# and CMFSin installed.

# You can get Zope from 'http://zope.org'
# You can get CMF from 'http://cmf.zope.org'
# You can get Plone from 'http://plone.org'
# You can get CMFBibliography (my stuff ;-) from
# 'http://www.neuroinf.de/LabTools'
# You can get CMFSin from
# 'http://sourceforge.net/projects/collective'


# For platform independent path construction we will need
# the 'path' module from the 'os' package

from os import path


# First of all, we define the name of the site that we are going
# to create (feel free to change it to whatever you like):

SITE_NAME = 'ScriptedSite'

# Next, we specify whether we want a CMFDefault or Plone site:

Plone = 0
Plone = 1   # comment out this line if you want a CMFDefault site.


# Here comes the top-level script. This is the one that has to be
# registered as external method to be called via Zope's
# management interface from the Zope root folder.

def script_site(self, site_id=SITE_NAME):
    """ the main script
    This has to be registered as external method from the Zope root
    """
    
    # 'self' is the Zope root
    # All functions called here are defined later in this module.
    
    setup_site(self, site_id)
    addUsers(self, site_id)
    addContent(self, site_id)
    
    # Up to here everything should work fine if you have
    # Zope/CMF/(Plone) out-of-the-box installed.
    #
    # For the following, you need to have additional products
    # installed.
    #
    # For best experience of this demo I recommend that you
    # get them following the links given above.
    
    try:
        addCMFBibliography(self, site_id)
    # I know it's bad style not to check for a particular
    # error but anyway
    except:
        pass
    try:
        addCMFSin(self, site_id)
    except:
        pass

def setup_site(self, id):
    """ add a CMF or Plone site to the Zope root """

    if Plone:
        self.manage_addProduct['CMFPlone'].manage_addSite(
            id,
            custom_policy='Default Plone')
    else:
        self.manage_addProduct['CMFDefault'].manage_addCMFSite(
            id)
    
    # The function 'manage_addSite' is found in
    # Products.CMFPlone.Portal.py

    # The function 'manage_addCMFSite' is found in
    # Products.CMFDefault.Portal.py

    # This is also invoked when you add a CMF or Plone site
    # through Zope's management interface.
    
    # Note that 'custom_policy' has to be set explicitly for
    # a Plone site.
    # (comment to Alan: wouldn't it be better to provide
    #  a default here as well?).

    
    
def getSite(self, site_id):
    """ get a handle on the portal site"""
    
    # This is just a little helper method to access the
    # portal object more easily.
    #
    # Remember: 'self' is the Zope root.
    
    return getattr(self, site_id)

    # This demonstrates the usage of Python's standard way
    # of getting at objects from their container when you
    # have their id stored as a string in a variable.
    # Otherwise 'self.ScriptedSite' (where you specify the
    # name hold in the variable explicitly) would do the
    # same thing.


def addUsers(self, site_id):
    """ add a few users to the site """

    # First, we need to get a handle on the site
    
    site = getSite(self, site_id)

    # There is a tricky issue now because 'addMember',
    # which we are going to call, needs a skin context
    # in the case of a Plone site in order to find
    # 'homePageText' (CMFPlone/skins/plone_scripts/
    # homePageText.py). This is a skin method that allows
    # you to customize the members' initial homepage.
    
    # Because the way in which we created and access
    # our site here does not provide a skin context
    # so far (the ZPublisher does not traverse to a
    # skinnable object manager which would trigger the
    # setup) we need to establish a skin context explicitly.

    # To this end we call (abuse)
    
    if Plone:
        try:
            site.changeSkin('Plone Default')
        except AttributeError:
            pass

    # a method of the 'SkinnableObjectManager'
    # defined in Products.CMFCore.Skinnable.py

    # Note: 'changeSkin' is relatively new (from CMF 1.4
    # on and prior only from CVS versions) therefore
    # the 'try - except'

    # Some versions of Plone on the other hand require
    # the skin context for 'addMember'. If you happen
    # to have a setting where the skin context is needed
    # but can't be set in this way, you can still patch
    # Plone's 'MembershipTool'.
    # Look for the 'createMemberarea' method and 
    # comment out the line where it says
    # DEFAULT_MEMBER_CONTENT = self.homePageText()
    # (or something comparable).
    
    # Next, we define a list of (id, name, password) tuples
    # to be iterated over afterwards
    
    members = [('joe_user', 'Joe User', 'abcde'),
               ('jane_bar', 'Jane Bar', 'vwxyz')]
    

    # Now we are ready to go
    
    for member_id, fullname, password in members:
        site.portal_registration.addMember(
            member_id
            , password
            , properties={ 'username': member_id
                           ,'fullname': fullname
                           , 'email' : 'foo@bar.com'
                           }
            )

        # 'addMember' is a method of the
        # 'RegistrationTool' class defined in 
        # Products.CMFCore.RegistrationTool.py
        
        # This is what's usually invoked by the skin
        # script 'register' (after some checkings)
        # (CMFDefault/skins/control/register.py or
        # CMFPlone/skins/plone_scripts/form_scripts/register.py)
        # which is called when someone submits the 'join_form'
        # (CMFDefault/skins/zpt_generic/join_form.pt or
        # CMFPlone/skins/plone_forms/join_form.pt)
        # which is made available by the 'Join' action
        # provided by 'portal_registration'.
        #
        # Confirm that you can login with these accounts.
        #
        # Note how things get involved here.
        

        # As of now, the homepages are owned by you
        # (or more precisely by the user who invokes
        # this script).
        #
        # To grant ownership of 'index_html' to the
        # newly created users we call a helper function
        # (defined at the end of this file).
        
        user_folder = getattr(site.Members, member_id)
        makeOwnedBy(user_folder.index_html, member_id)

        #
        # That's it for the users part.
        #
        

def addContent(self, site_id):
    """ add, publish, and discuss some content """
    
    # Now, we are going to add a news item and a document
    # to Joe's user folder. The news item will get published
    # and Jane will add a comment to it.
    # Then we will create a topic in the site's root folder.
    
    
    # This is what will go in the news item:

    lead_in="A new kid in town."
    
    news_body="""
    This is the body of the scripted news item\n
    \n
    Hello World!\n\n
        I just joined.\n\n
    Joe User
    """
    
    # So let's go:
    
    site = getSite(self, site_id)
    joes_folder = site.Members.joe_user
    
    # Create an empty news item in Joe's member folder
    
    joes_folder.invokeFactory('News Item', 'joined')

    # 'invokeFactory is a method of the 'PortalFolder' class
    # defined in Products.CMFCore.PortalFolder.py
    # (which is inherited via the 'SkinnedFolder' class by the
    # 'PloneFolder' class which 'joes_folder' is an instance of
    # in case of a Plone site).

    # 'invokeFactory' will not only create the news item but also
    # notify the 'portal_catalog' and the 'portal_workflow'
    # about its existence.
    # This is why you should use this method as opposed to calling
    # the news item's constructor (the function 'addNewsItem' from
    # Products.CMFDefault.NewsItem.py) directly.
    # By the way: This would happen if you created any portal
    # content object through Zope's management interface; so don't
    # do that.
    
    # Get an object handle on the news item.
    
    doc = getattr(joes_folder, 'joined')
    
    # Alternatively we could call
    # doc = joes_folder.joined
    # because we specify the id explicitly here.
    # Note the difference in the syntax though.
    
    # Make it owned by Joe.

    makeOwnedBy(doc, 'joe_user')
    
    
    # Change the title and add some further meta data.
    
    # This is to demonstrate how you can set individual
    # meta data fields explicitly.
    
    doc.setTitle('Hello World')
    doc.setSubject(['executable documentation', 'scripting Zope'])

    # These and other meta data mutator methods are provided by
    # the 'DefaultDublinCoreImpl' mix-in class defined in
    # Products.CMFDefault.DublinCore.py
    # All classes defining portal content objects inherit this.
    # It is called a mix-in class because it does not define
    # any objects but just a bunch of attributes and methods
    # to be made available to other objects through inheritance.
     
    # Next, we fill the body.
    
    doc.edit(news_body,
             description=lead_in,
             text_format='structured-text')
    
    # 'edit' is a method of the 'NewsItem' class defined in
    # Products.CMFDefault.NewsItem.py
    
    
    # Now let's manipulate it a bit by making use of some tools
    # (again, note how the tools are addressed explicitly):
    

    # 1. Allow this doc to be discussed. For news items this is not
    # the default, so we change this explicitly here.
    
    doc.portal_discussion.overrideDiscussionFor(doc,1)

    # 'overrideDiscussionFor' is a method of the 'DiscussionTool' class
    # defined in Products.CMFDefault.DiscussionTool.py
    
    # Note: Starting this call with 'doc' we hook ourselfs deep down
    # into the ZODB hierarchy, i.e., we start at
    # /ScriptedSite/Members/joe_user/joined
    # from where 'portal_discussion' is found in 'ScriptedSite'
    # through acquisition.
    # Alternatively, we could have used 'site.portal_discussion...'
    # as demonstrated below for the workflow tool or we could have
    # imported 'getToolByName' from Products.CMFCore.utils but
    # for the sake of simplicity of this script I want to get around
    # without imports here.
        

    # 2. Do a workflow action: Publish it.
    
    site.portal_workflow.doActionFor(
        doc,
        'publish',
        comment="Let's make it appear in the news listing")

    # 'doActionFor' is a method of the 'WorkflowTool' class
    # defined in Products.CMFCore.WorkflowTool.py

    # Now this news item will appear in the news listing
    # available via the portal tab called 'news'.
    

    # 3. Add a comment:
    
    talkback = site.portal_discussion.getDiscussionFor(doc)
    talkback.createReply(title='Hi Joe!'
                         , text='Welcome to the club. \n \n Jane'
                         , Creator='Jane Bar'
                         )

    # 'getDiscussionFor' is a method of the 'DiscussionTool' class
    # defined in Products.CMFDefault.DiscussionTool.py which
    # creates (if needed) and returns the appropriate 'talkback'
    # which is an instance of the 'DiscussionItemContainer' class
    # defined in Products.CMFDefault.DiscussionItem.py providing
    # the 'createReply' method.

    
    # Creating a news item as shown here demonstrates that usually
    # you go through a two step process. First you create an 'empty'
    # object of the desired type and in a second step you
    # 'fill' it by calling its edit or other mutator methods.
    # The web interface to content creation in CMF/Plone
    # reflects this directly.

    # Most default content types, however, also allow for
    # creation in one step. You just pass appropriately
    # named arguments to 'invokeFactory' as demonstrated
    # for a document now.

    # This document will have a more substantial content so we
    # read it from a file instead of defining it here in place.

    # To get at the file that we want to read in, we first figure
    # out where we live in the file system:
    
    instance_home = self.Control_Panel.getINSTANCE_HOME()
    
    # (/the/path/to/your/zope in the file system)

    # 'getINSTANCE_HOME' is a method of the 'ApplicationManager'
    # class which the 'Conrol_Panel' is an instance of.
    # The 'ApplicationManager' is not CMF or Plone specific
    # but a standard Zope class defined in
    # App.ApplicationManager.py
    # Note the missing 'Product' in the beginning. To find
    # its source, go to '/the/path/to/your/zope/lib/python/App/.
    
    # If the installation instructions that come with this script
    # have been followed correctly, we will find the content in

    filepath = path.join(instance_home,
                         'Extensions',
                         'ScriptingResources',
                         'BeginnersGuide.txt')

    # Note the usage of Pythons 'path' function from the
    # 'os' module to make this work platform independently.

    # We open it and read in the content.
    
    doc_body = open(filepath,'r').read()

    # Now we have all we need to create the document.
    
    joes_folder.invokeFactory('Document',
                              'BeginnersGuide',
                              text_format='structured-text',
                              text=doc_body)

    # As before, we set a few meta data and transfer the
    # ownership to Joe.
    
    doc = getattr(joes_folder, 'BeginnersGuide')

    theTitle = "A Beginner's Guide to the World of Zope"
    doc.setTitle(theTitle)

    theDescription  = "Just a loose collection of commented links "
    theDescription += "to things I found useful for getting around "
    theDescription += "with Python, Zope, CMF, and Plone."
    doc.setDescription(theDescription)

    doc.setSubject(['documentation', 'Zope', 'CMF', 'Plone'])

    makeOwnedBy(doc, 'joe_user')


    # To illustrate the usage of a more advanced content type
    # we are now going to create a topic. Say we want to have
    # a section 'Documentation' in our portal where everything
    # related to documentation should show up irrespective of
    # its content type or location in the object hierarchy (i.e.,
    # where it is in the ZODB). We establish the relation here
    # by using the meta data 'Subject' field.

    # To do this, we go through three steps:


    # 1. Create an empty topic called 'Documentation'.
    
    site.invokeFactory('Topic',
                       'Documentation',
                       title='Documentation')

    # Get a handle on it.
    
    topic = getattr(site, 'Documentation')


    # 2. Add a criterion on the 'Subject' field of type 'List Criterion'.
    
    topic.addCriterion(field='Subject',
                       criterion_type='List Criterion')

    # 'addCriterion' is a method of the folderish 'Topic' class
    # defined in Products.CMFTopic.Topic.py
    # Depending on the 'criterion_type' specified, it generates
    # an instance of the appropriate criterion class in 'topic'.
    # Here, this is a 'ListCriterion' as defined in
    # Products.CMFTopic.ListCriterion.py
    # Possible field values are all existing catalog indexes.
    # Browse the topic's web interface or CMFTopic's source code
    # to explore further options.

    # Get a handle on the criterion.
    
    criterion = getattr(topic, 'crit__Subject')

    # Note: we did not specify the id used here before.
    # It has been auto-generated by 'addCriterion' through
    # concatenation of 'crit__' with the field value.


    # 3. Specify the keywords defining our topic.
    
    criterion.edit(value=["documentation", "executable documentation"])

    # 'edit' is a method of the 'ListCriterion' class defined in
    # Products.CMFTopic.ListCriterion.py
    

    #
    # That's it for the 'addContent' part.
    #
    

def addCMFBibliography(self, site_id):
    """ install the CMFBibliography product """
    
    # To this end, we need to create an external method first
    # and invoke it then.
    # This is what you usually do through the ZMI or what
    # the quick installer would do for you.

    # Get the site.
    
    site = getSite(self, site_id)
    
    # Create the method ...
    
    site.manage_addProduct['ExternalMethod'].manage_addExternalMethod(
        'install_bibliography',
        'from www.neuroinf.de/LabTools',
        'CMFBibliography.Install',
        'install')
    
    # ... and invoke it.
    
    site.install_bibliography()
    
    # Note the parentheses in the end. Without those we would just
    # have a reference to the method without calling it.

    # 'manage_addExternalMethod' is a function of the 'ExternalMethod'
    # module defined in Products/ExternalMethod/ExternalMethod.py
    
    
    # Now let's use the product.
    #
    # Add a 'Bibliography Folder' to Jane's home folder ...
    
    janes_folder = site.Members.jane_bar
    janes_folder.invokeFactory('Bibliography Folder', 'my_references')
    bibfolder = getattr(janes_folder, 'my_references')
    makeOwnedBy(bibfolder, 'jane_bar')
    
    # .. and upload some content from a file.

    # To this end we need to specify where to look for
    # the file and how it is called.
    #
    # Because this time we need something from a file system based
    # product we start from the software_home ...
    
    sw_home = self.Control_Panel.getSOFTWARE_HOME()
    filepath = path.join(sw_home,
                         "Products",
                         "CMFBibliography",
                         "upload_samples",
                         "bibtex_sample.bib")
    
    # ... to specify the path in such a way that it will
    # (hopefully) work irrespective of the local system.


    # Now that we know what and where to look for
    # we do it:
    
    infile = open(filepath, "r")
    source = infile.read()
    infile.close()  # we are nice citizens ;-)
    
    # A note aside: Actually I don't know whether
    # source = open(filepath, "r").read()
    # would leave the file handle open or not.
    # Look at Python's garbage collector if you want to
    # find out yourself.
    

    # Now we are ready to call bibliography folders'
    # upload method:
    
    bibfolder.processUpload(source,"bibtex_sample.bib")

    # 'processUpload' is a method of the 'BibliographyFolder' class
    # defined in Products.CMFBibliography.BibliographyFolder.py
    
    
    # Again, we transfer ownership of all uploaded
    # items to Jane (after tweaking the meta data a bit).
    
    for obj in bibfolder.contentValues():
        
        # 'contentValues()' is the CMF way of getting a
        # list of child objects from a portal folder.
        # It provides a filtered view onto 'objectValues()',
        # the "Zope way" of listing all children of a folder,
        # allowing only PortalFolders and
        # PortalContent-derivatives to show through.
        # It is a method of the 'PortalFolder' class defined in
        # Products.CMFCore.PortalFolder.py


        # Set the subject to a value that will be picked
        # up by the 'Documentation' topic.
        
        obj.setSubject(["documentation"])

        # Note the syntax here: we are passing in a list with one
        # element and this element is a string.

        # To avoid the lengthy description entries in search
        # results and topic views we cut the description
        # back to 80 characters in length.
        #
        # A note aside: Don't do this in a serious application
        # in this way. The 'Description' field is important in
        # various ways and its value should not be influenced
        # by those styling issues. If you need something like that
        # customize your skins, e.g., introduce a new property
        # like 'short description' and display this in overview
        # templates.
        
        abstract = obj.getProperty('abstract', 'no abstract available')
        if (len(abstract) > 80):
            obj.setDescription(abstract[:80] + "...")

        # 'getProperty' is a method of the 'PropertyManager' class
        # defined in OFS.PropertyManager.py which is inherited by all
        # portal content. The second argument provides a default
        # value in case the property does not exist.
        

        # Now transfer the ownership. A side effect of doing this
        # as the last modification in the loop is that the catalog
        # will reflect all changes made to the meta data above
        # without explicitly calling 'obj.reindexObject()' because
        # 'makeOwnedBy()' does this anyway in the end.
        
        makeOwnedBy(obj, 'jane_bar')
        

    #
    # That's it for the bibliography part.
    #
        

def addCMFSin(self, site_id):
    """ install the CMFSin product """
    
    # The installation works the same way as before:

    # Get the site ...
    
    site = getSite(self, site_id)
    
    # ... create the install method ...
    
    site.manage_addProduct['ExternalMethod'].manage_addExternalMethod(
        'install_cmfsin',
        'from http://sourceforge.net/projects/collective',
        'CMFSin.Install',
        'install')
    
    # ... and invoke it to install CMFSin.
    
    site.install_cmfsin()
    

    # Now we want to configure the tool a bit:
    
    # Again, we need the content of a file.
    # This time it is again located in a subdirectory
    # of the 'Extensions' folder.
    
    instance_home = self.Control_Panel.getINSTANCE_HOME()
    filepath = path.join(instance_home,
                         "Extensions",
                         "ScriptingResources",
                         "sin_demo.cfg")
    
    cfg_content = open(filepath,'r').read()

    # Set the configuration as specified in the file:
    
    site.sin_tool.manage_configSin(config=cfg_content,
                                   submit='Set Config')

    # 'manage_configSin' is a method of the 'SinTool' class
    # defined in Products.CMFSin.SinTool.py
    # 'submit' can be set to 'Import', 'Export', or 'Set Config'
    # to trigger different actions.
    # In the web interface this is reflected by different
    # buttons with the respective labels.

    
    # Finally, add the 'zope_news' slot to the right-hand side
    # of the portal's homepage by setting the value of 'right_slots'
    # completely anew here. This will only affect Plone sites since
    # CMFDefault sites don't provide this property.

    site.manage_changeProperties(
        {'right_slots':['here/workflow_review_slot/macros/review_box',
                        'here/calendar_slot/macros/calendarBox',
                        'here/events_slot/macros/eventsBox',
                        'here/sin_tool/macros/zope_news'
                        ]
         })

    # 'manage_changeProperties' is another method of the
    # 'PropertyManager' mentioned above.
    
    # Alternatively, we could just add the last line to the
    # existing property by calling
    # r_slots = site.getProperty('right_slots', [])
    # r_slots.append('here/sin_tool/macros/zope_news')
    # site.manage_changeProperties({'right_slots':r_slots})


    # This demonstrates the standard way of using a method from
    # core Zope's 'PropertyManager' which all portal content
    # inherits from.
    #
    # When calling 'manage_changeProperties' you can either
    # pass in the REQUEST or a dictionary with
    # key (property name) value (what to set it to) pairs.
    #
    # See the Zope API documentation or the source code in the
    # 'ObjectFileSystem' folder ('ZOPEHOME/lib/python/OFS/')
    # for more.

    #
    # That's it for the syndication part.
    #
    
        
# We still need to define the helper function 'makeOwnedBy' 
# that we called a couple of times:

def makeOwnedBy(obj, new_owner_id):
    """ Transfer ownership to the user specified by the
        'new_owner_id'.
    (the 'executable owner' as well as the 'Owner' local role)
    
    WARNING: Don't make a function like this available to
    regular users. This could open up a security hole if you
    provide your users with ways to generate executable content.
    Zope (as opposed to CMF/Plone) usually does so, e.g., by
    allowing the manipulation of DTML, ZPTs, Scripts(Python)
    and the like. 
    Therefroe, in plain Zope the 'executable owner' is crucial to
    prevent so-called trojan-horse attacks where low priviledged
    users enter nasty code that they are not allowed to execute
    but they succeed in making a more priviledged user execute it.
    For further background information on security issues see the
    chapters on security in the Zope book and the Zope developers
    guide or read
    http://zope.org/Members/mcdonc/PDG/6-1-Security.stx/view
    (don't worry that it's 'old'; the basics are still relevant.)
    """
    
    # Get a handle on the user object of the new owner
    # ('acl_users' is found through acquistion).
    
    new_owner_obj = obj.acl_users.getUser(new_owner_id)

    # 'getUser' is a method of the 'UserFolder' class defined in
    # AccessControl.User.py
    
    # Wrap it in the context of the user folder
    # to add the security info that is necessary for
    # setting the 'executable owner'.
    
    new_owner_obj = new_owner_obj.__of__(obj.acl_users)
    
    # Note:
    # new_owner_obj = obj.portal_membership.getMemberById(new_owner_id)
    # would not help us because that way the user object returned
    # would still be missing the necessary security information.

    # Now, let's first transfer the 'executable ownership':
    
    obj.changeOwnership(new_owner_obj)

    # 'changeOwnership' is a method of the 'Owned' class defined in
    # AccessControl.Owned.py which (almost?) all higher level Zope
    # objects inherit from.
    
    # Note:
    # In case you are wondering why we need to do that at all
    # even so CMF/Plone has no concept of 'executable content'.
    # This is because the Dublin Core 'Creator()' method returns
    # 'self.getOwner()' which finds the 'executable owner'.

    # But we are not done yet: other tools like the workflow tool
    # or the actions tool don't look at the 'executable owner' but
    # rather the 'Owner' local role. So we also need to do:
    
    obj.manage_setLocalRoles(new_owner_id, ['Owner'])

    # 'manage_setLocalRoles' is a method of the 'RoleManager' class
    # defined in AccessControl.Role.py
    
    # To be on the save side, let the catalog know that things have
    # changed:
    
    obj.reindexObject()

    # 'reindexObject()' is a method of the 'CMFCatalogAware' class
    # defined in Products.CMFCore.CMFCatalogAware.py which all
    # classes defining portal content objects inherit from. 


#
#  Thanks for reading all the way to here.
#  I hope you find this helpful.
#
#  Feedback (comments, bug reports, feature requests, ...)
#  are always welcome at r.ritz@biologie.hu-berlin.de
#
#  Raphael
#


# Where to go from here:

# This example is meant to illustrate how to do things from scripts
# that one usually get's to know through the web interface in the
# first place. It is by no means an extensive demonstration of
# Zope/CMF/Plone's API. It is just a (hopefully) working example
# to help you getting started with scripting Zope/CMF/Plone.

# If you need to find out yourself how something particular is done
# you should look at the source code of the appropriate module and
# there at the tests and interfaces (if present) in the first place.
#
#   Don't be afraid of doing so!
#
# It is one of the major benefits of Python that the source code is quite
# readable (as compared to other programming languages at least).

# Furthermore, Zope/CMF/Plone source code is often well commented,
# as you've seen here ;-)
