#! /usr/bin/env python

#
# rscript_site.py
#

# part of ScriptingCMF (version 0.1)
# copyright: ITB, Humboldt-University Berlin, 2003
# written by Raphael Ritz, September 2003


# First, the general stuff to make
# some modules, user credentials
# and the server available

import sys
from os import path
import urllib

# a shortcut
encode = urllib.urlencode

protocol = "http://"

# get the user credentials
# Note: For safty reasons I recommend to generate
# a temporary account with management priviledges
# for exploring this script because basic http
# authentication is used which can easily be sniffed.

print "\nusername (needs to have the 'Manager' role): ",
user = sys.stdin.readline()[:-1]
print "password: ",
passwd = sys.stdin.readline()[:-1]
admin = "%s:%s@" % (user, passwd)

print "servername (default: 'localhost'): ",
s_name = sys.stdin.readline()[:-1]
if s_name: server_name = s_name
else: server_name = "localhost"

print "port (default: '8080'): ",
p_name = sys.stdin.readline()[:-1]
if p_name: port = ':' + p_name
else: port = ":8080"

server = protocol + server_name + port
admin_at_server = protocol + admin + server_name + port


# Some names (aka ids)

site_id      = "RemoteDemoSite"
news_item_id = "Hello"
document_id  = "BeginnersGuide"
topic_id     = "Documentation"


# enabling different modes

# By in- and outcommenting the following
# lines you control the interactivity, the
# logging mode, and whether we make a CMFDefault
# or Plone site.

# interactive = 0   # no questions asked
interactive = 1   # waits after every call

Plone = 0         # a CMFDefault site
Plone = 1         # a Plone site

#log_mode = 0  # no logs
log_mode = 1  # print to stdout
#log_mode = 2  # write to file


if log_mode and log_mode != 1:
    logFile = open('rscript.log','w')


# some utility methods

def log_it(url, params=None, comment=''):
    if not log_mode: return None
    else:
        output = "Calling " + url + '\n'
        if params: output += "with parameters " + params + '\n'
        if comment: output += '\n' + comment + '\n\n'

    if log_mode == 1:
        print output
    else:
        logFile.write(output)

def call(url, params=None, comment=''):
    urllib.urlopen(url,params)
    log_it(url, params, comment)
    if interactive: 
        print "\nPress 'Return' to continue."
        sys.stdin.readline()


##
##  Now, we are ready to go :-)
##
        
# First, make a portal site:

if Plone:
    constructor = "/manage_addProduct/CMFPlone/manage_addSite"
    input = encode({'id':site_id,
                    'custom_policy':'Default Plone'
                    })
    comment = "created a Plone site at %s/%s." % (server, site_id)

else:
    constructor = "/manage_addProduct/CMFDefault/manage_addCMFSite"
    input = encode({'id':site_id})
    comment = "created a CMFDefault site at %s/%s." % (server, site_id)
    

url = admin_at_server + constructor

call(url, input, comment)


# Second, add a few users:

constructor = '/' + site_id + "/register"

members = [('joe_user', 'Joe User', 'abcde'),
           ('jane_bar', 'Jane Bar', 'vwxyz')]

for member_id, fullname, password in members:
    
    input = encode({
        'fullname': fullname,
        'username': member_id,
        'password': password,
        'confirm': password,
        'email': 'foo@bar.com'
        })
    
    comment = "registered '%s'." % fullname
    
    call(server + constructor, input, comment)
    
    # Note that this is called anonymously.


# For later use we define
joe_at_server  = protocol + 'joe_user:abcde@' + server_name + port
jane_at_server = protocol + 'jane_bar:vwxyz@' + server_name + port


# Next, Joe adds a news item and a document to his folder.

joes_folder = '/' + site_id + '/Members/joe_user/'
folder = joe_at_server + joes_folder

constructor = 'invokeFactory'
input = encode({'type_name':'News Item',
                'id':news_item_id
                }
               )

comment = "created an empty news item called '%s'." % news_item_id

call(folder + constructor, input, comment)


# Set some meta data

doc = folder + news_item_id

comment = "set the title."

call(doc + "/setTitle?title=Hello+World", comment=comment)
# note how the urlencoding is done explicitly here

input = encode({'subject':['executable documentation',
                           'scripting Zope']},1)
# 'doseq' (the second argument) needs to be true
# in order to get this accross as a sequence

comment = 'set two keywords in subject.'

call(doc + "/setSubject", input, comment)

# finaly, fill the lead in and the body

lead_in = "A new kid in town."

news_body="""
This is the body of the scripted news item\n
\n
Hello World!\n\n
    I just joined.\n\n
Joe User
"""

input = encode({'text_format': 'structured-text'
                ,'text': news_body
                , 'description': lead_in
                })

comment = "filled the body text and specified the format."

call(doc + "/edit", input, comment)

# Allow it to be discussed.
comment = "made '%s' discussable." % news_item_id

call(doc + "/metadata_edit?allowDiscussion=on", comment=comment)

# Why does this cancel the subject entries?
# In case of no specified subjects it should get them via 'obj.Subject()'.
# Alternatively: How does one call something like 'overrideDiscussionFor'
# where a handle of the object needs to be passed in explicitly?
# Anyway: Let's set the subject again.

input = encode({'subject':['executable documentation',
                           'scripting Zope']},1)
comment = "called again to fix the canceling through the 'allowDiscussion' call."

call(doc + "/setSubject", input, comment)

comment="to let the catalog know that things have changed."
call(doc + "/reindexObject", comment=comment)


# Joe submits it for publication.

input = encode({'workflow_action':'submit',
                'comment':'Let the world know I joined.'
                })

comment = "Joe submitted '%s' for publication" % news_item_id

call(doc + "/content_status_modify", input, comment)


# admin publishes it.

input = encode({'workflow_action':'publish',
                'comment':'Welcome!'
                })

comment = "%s published '%s'" % (user, news_item_id)
comment += "\nThis makes it appear in the news listing also."

call(admin_at_server + joes_folder + news_item_id +
     "/content_status_modify", input, comment)


# Next, let's Jane comment on it.

# The following does not work but I am stuck here because I don't know
# whether this is due to a limitation of urllib (not supporting HTTP 1.1)
# or me being unable to keep the connection alive
# or anything else I don't see. Or maybe it does ...?

input = encode({'title':'Hi Joe',
                'text':'Welcome to the club!\n\n Jane',
                'Creator':'Jane',
                'discussion_reply:method':'Add Comment'
                })

comment = "Jane should have added a comment but this does not work yet."

call(jane_at_server + joes_folder + news_item_id + "/talkback",
     input, comment)

comment = "But if we repeat the call ..."

call(jane_at_server + joes_folder + news_item_id + "/talkback",
     input, comment)


# The document will have a more substantial content so we
# read it from a file instead of defining it here in place.

filepath = path.join('ScriptingResources',
                     'BeginnersGuide.txt')

doc_body = open(filepath,'r').read()


joes_folder = '/' + site_id + '/Members/joe_user/'
folder = joe_at_server + joes_folder

constructor = 'invokeFactory'
input = encode({'type_name':'Document',
                'id':document_id,
                'title': "A Beginner's Guide to the World of Zope",
                'description': "A loose collection of ...",
                'text_format':'structured-text',
                'text':doc_body[:500]
                }
               )

comment = "created an empty (why is that?) document called '%s'." % document_id

call(folder + constructor, input, comment)

# Because passing additional arguments to invoke factory
# does not work in this context we need to do more work:

# Set some meta data

doc = folder + document_id
theTitle = "A Beginner's Guide to the World of Zope"
input = encode({'title':theTitle})
comment = "set the title."

call(doc + '/setTitle', input, comment)

theDescription  = "Just a loose collection of commented links "
theDescription += "to things I found useful for getting around "
theDescription += "with Python, Zope, CMF, and Plone."
input = encode({'description':theDescription})
comment = "set the description."

call(doc + '/setDescription', input, comment)

input = encode({'subject':['documentation',
                           'Zope',
                           'CMF',
                           'Plone']},1)
comment = 'set some keywords in subject.'

call(doc + "/setSubject", input, comment)

input = encode({'text_format': 'structured-text'
                ,'text': doc_body})

comment = "filled the body text and specified the format."

call(doc + "/edit", input, comment)

# Next we make a topic in the site root

admin_at_site = admin_at_server + '/' + site_id
constructor = "/invokeFactory"
url = admin_at_site + constructor

input = encode({'type_name':'Topic',
                'id':topic_id})
comment = "created an empty topic called '%s'." % topic_id

call(url, input, comment)

topic = admin_at_site + '/' + topic_id
comment = "set the title."
call(topic + "/setTitle?title=Documentation", comment=comment)

input = encode({'field':'Subject',
                'criterion_type':'List Criterion'})
comment = "added a criterion."
call(topic + "/addCriterion", input, comment)

input = encode({'value':["documentation", "executable documentation"]},
                1)
comment = "set the values for the criterion."
call(topic + "/crit__Subject/edit", input, comment)


# Now install CMFBibliography

# To that end, first create an external method:
constructor = "/manage_addProduct/ExternalMethod/manage_addExternalMethod"

admin_at_site = admin_at_server + '/' + site_id
url = admin_at_site + constructor

input = encode({'id':'install_bibliography',
                'title':'from www.neuroinf.de/LabTools',
                'module':'CMFBibliography.Install',
                'function':'install'
                })

comment = "created an external method to install 'CMFBibliography'"

call(url, input, comment)

# Now this external method needs to be called in order to
# do the installation.
url = admin_at_site + '/install_bibliography'
comment = "did the installation."

call(url, comment=comment)


# Jane makes a BibliographyFolder and uploads some content.

janes_folder = '/' + site_id + '/Members/jane_bar/'
bibfolder_name = 'my_references'

url = jane_at_server + janes_folder + 'invokeFactory'
input = encode({'type_name':'Bibliography Folder',
                'id':bibfolder_name})

comment = "makes Jane create a 'Bibliography Folder'."
call(url, input, comment)


# Open the file to be uploaded.
# For this to work the 'ScriptingResources' directory needs
# to be in the same folder as this script

filepath = path.join('ScriptingResources',
                     'bibtex_sample.bib')

infile = open(filepath, "r")
source = infile.read()
infile.close()

url = jane_at_server + janes_folder + bibfolder_name +'/processUpload'

input = encode({'source':source,
                'filename':'bibtex_sample.bib'
                })

comment = "uploads some references.\n\n"
comment += "Check out the new portal tab 'bibliography'\n"
comment += "to see the consequences of this."

call(url, input, comment)


# Install CMFSin

constructor = "/manage_addProduct/ExternalMethod/manage_addExternalMethod"
admin_at_site = admin_at_server + '/' + site_id
url = admin_at_site + constructor

input = encode({'id':'install_cmfsin',
                'title':'from http://sourceforge.net/projects/collective',
                'module':'CMFSin.Install',
                'function':'install'
                })

comment = "created an external method to install 'CMFSin'."

call(url, input, comment)


url = admin_at_site + '/install_cmfsin'

comment = "did the installation of the sin tool."

call(url, comment=comment)


# now we configure CMFSin
filepath = path.join('ScriptingResources',
                     'sin_demo.cfg')
config_file = open(filepath,'r')
cfg = config_file.read()
config_file.close()

url = admin_at_site + '/sin_tool/manage_configSin'

input = encode({'config:text':cfg,
                'submit':'Set Config'
                })

comment = "configures the channels and maps."

call(url, input, comment)


# Finaly, we add the zope_news slot to the site.
# This demonstrates the update of a portal property

url = admin_at_site + '/manage_changeProperties'

input = encode({'right_slots':['here/workflow_review_slot/macros/review_box',
                               'here/calendar_slot/macros/calendarBox',
                               'here/events_slot/macros/eventsBox',
                               'here/sin_tool/macros/zope_news'
                               ]},1)

comment = "added 'zope news' to the slots on the right-hand site."

call(url, input, comment)

if interactive:
    print "I hope you enjoyed the demo.\n"
    print "Feedback welcome at r.ritz@biologie.hu-berlin.de\n"
    print "For an alternative approach and more detailed comments"
    print "see the accompaining module 'ScriptSite.py'."
    print "\n    Raphael \n"

# be a nice citizen
if log_mode and log_mode != 1:
    logFile.close()
